/*********************************************************************/
/*  File Name : ACTLEN.sas                                           */
/*  Date      : 2016-07-16                                           */
/*  Auther    : Masahito Katayama  SHIONOGI & CO., LTD.              */ 
/*********************************************************************/

/* Libname */
libname SDTM "X:XXX\XXXXX\XXXXXX\XXXXXXX\tabulations\data" access=readonly;

options mprint symbolgen mlogic source source2 linesize=200 ;

/*
 Macro parameter 
 InLIB : The libname of sas datasets which you want to shrink
 FPATH : The File path of xpt files
*/

%macro ACTLEN(InLIB= ,FPATH = ) ;
  DATA __VARS;
    SET SASHELP.VCOLUMN(where=(libname=upcase("&inLIB.") & index(MEMNAME,"_")=0)) ;
    call symput(cats("ds"  ,_n_),MEMNAME);
    call symput(cats("var" ,_n_),NAME);
    call symput(cats("type",_n_),TYPE);
    ID=_N_;
    KEEP LIBNAME MEMNAME NAME LENGTH TYPE FORMAT ID;
  RUN;
  PROC SQL NOPRINT;
    CREATE TABLE __VARSX AS 
    SELECT __VARS.*,
         VTABLE.LIBNAME,  VTABLE.MEMNAME,  VTABLE.MEMLABEL
    FROM __VARS LEFT JOIN SASHELP.VTABLE
    ON __VARS.LIBNAME=VTABLE.LIBNAME AND __VARS.MEMNAME=VTABLE.MEMNAME
    ORDER BY ID ;
  QUIT;

  /* Counting number of variables in the datasets */
  PROC SQL NOPRINT;
    SELECT COUNT (*)INTO :NVARS FROM __VARSX;
  QUIT;
  /* Computing max. length of each variable in the dataset */
  %DO i = 1 %TO &NVARS.;
     %IF( %UPCASE ("&&TYPE&I..")= "CHAR") %THEN %DO; 
       PROC SQL NOPRINT;
         SELECT MAX (LENGTHN(&&var&i..)) INTO :mlen&i. FROM %trim(&inLIB.).%trim(&&ds&i..);
       QUIT;
     %END; %ELSE %DO;
       %LET mlen&i.= 8;
     %END;
   %END;

  /* Specifying maximum length into contents datasets */
  DATA __VARSX; SET __VARSX;
    %DO I = 1 %TO &NVARS.;
    IF(ID = &i.) THEN DO; MAXLEN=   &&mlen&i..; END;
  %END;
  RUN;
  /* Computing maximum length of the variable used across all datasets in inLIB */
  PROC SORT DATA = __VARSX; BY NAME DESCENDING MAXLEN; RUN;
  DATA __VARSX;  SET __VARSX;
    RETAIN ACTLEN;
    BY NAME DESCENDING MAXLEN;
    IF ( FIRST.NAME= 1) THEN ACTLEN= MAXLEN;
  RUN;
  PROC SORT DATA = __VARSX; BY MEMNAME ID; RUN;
  DATA _NULL_;  SET __VARSX;
    BY MEMNAME ID;
    IF (FIRST.MEMNAME=1) THEN DO;
      call execute("LIBNAME TEMPXPT XPORT '"|| "&FPATH." ||strip(MEMNAME)||".xpt';");
      call execute("PROC SQL NOPRINT;");
      call execute("CREATE TABLE "|| "tempxpt."|| strip(MEMNAME)||"(LABEL="||strip(MEMLABEL)||") AS");
      call execute("SELECT");
    END;
    IF (ACTLEN > 0) THEN DO;
      IF (LAST.MEMNAME=1) THEN DO;
        call execute( strip(NAME)||" LENGTH="||strip(put(ACTLEN,BEST.)) );
      END; ELSE DO;
        call execute( strip(NAME)||" LENGTH="||strip(put(ACTLEN,BEST.))||",");
      END;
    END;
    IF (LAST.MEMNAME=1) THEN DO;
      call execute("FROM "|| strip(LIBNAME) ||"."|| strip(MEMNAME) ||";");
      call execute("QUIT;");
    END;
  RUN;
  PROC DATASETS LIBRARY= WORK; DELETE __:; RUN; QUIT;
%mend;
%ACTLEN(InLIB=SDTM ,FPATH=%str(X:XXX\XXXXX\XXXXXX\XXXXXXX\tabulations\sdtm)) ;
